Coverage for F:\DEV\EMIZ\emiz\mission.py: 36%

# Coverage for F:\DEV\EMIZ\emiz\mission.py : 36%

## 874 statements 311 run 563 missing 0 excluded

Hot-keys on this page

r m x p toggle line displays

j k next/prev highlighted chunk

0 (zero) top of page

1 (one) first highlighted chunk

1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 552 553 554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 620 621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 652 653 654 655 656 657 658 659 660 661 662 663 664 665 666 667 668 669 670 671 672 673 674 675 676 677 678 679 680 681 682 683 684 685 686 687 688 689 690 691 692 693 694 695 696 697 698 699 700 701 702 703 704 705 706 707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 722 723 724 725 726 727 728 729 730 731 732 733 734 735 736 737 738 739 740 741 742 743 744 745 746 747 748 749 750 751 752 753 754 755 756 757 758 759 760 761 762 763 764 765 766 767 768 769 770 771 772 773 774 775 776 777 778 779 780 781 782 783 784 785 786 787 788 789 790 791 792 793 794 795 796 797 798 799 800 801 802 803 804 805 806 807 808 809 810 811 812 813 814 815 816 817 818 819 820 821 822 823 824 825 826 827 828 829 830 831 832 833 834 835 836 837 838 839 840 841 842 843 844 845 846 847 848 849 850 851 852 853 854 855 856 857 858 859 860 861 862 863 864 865 866 867 868 869 870 871 872 873 874 875 876 877 878 879 880 881 882 883 884 885 886 887 888 889 890 891 892 893 894 895 896 897 898 899 900 901 902 903 904 905 906 907 908 909 910 911 912 913 914 915 916 917 918 919 920 921 922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 937 938 939 940 941 942 943 944 945 946 947 948 949 950 951 952 953 954 955 956 957 958 959 960 961 962 963 964 965 966 967 968 969 970 971 972 973 974 975 976 977 978 979 980 981 982 983 984 985 986 987 988 989 990 991 992 993 994 995 996 997 998 999 1000 1001 1002 1003 1004 1005 1006 1007 1008 1009 1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 1027 1028 1029 1030 1031 1032 1033 1034 1035 1036 1037 1038 1039 1040 1041 1042 1043 1044 1045 1046 1047 1048 1049 1050 1051 1052 1053 1054 1055 1056 1057 1058 1059 1060 1061 1062 1063 1064 1065 1066 1067 1068 1069 1070 1071 1072 1073 1074 1075 1076 1077 1078 1079 1080 1081 1082 1083 1084 1085 1086 1087 1088 1089 1090 1091 1092 1093 1094 1095 1096 1097 1098 1099 1100 1101 1102 1103 1104 1105 1106 1107 1108 1109 1110 1111 1112 1113 1114 1115 1116 1117 1118 1119 1120 1121 1122 1123 1124 1125 1126 1127 1128 1129 1130 1131 1132 1133 1134 1135 1136 1137 1138 1139 1140 1141 1142 1143 1144 1145 1146 1147 1148 1149 1150 1151 1152 1153 1154 1155 1156 1157 1158 1159 1160 1161 1162 1163 1164 1165 1166 1167 1168 1169 1170 1171 1172 1173 1174 1175 1176 1177 1178 1179 1180 1181 1182 1183 1184 1185 1186 1187 1188 1189 1190 1191 1192 1193 1194 1195 1196 1197 1198 1199 1200 1201 1202 1203 1204 1205 1206 1207 1208 1209 1210 1211 1212 1213 1214 1215 1216 1217 1218 1219 1220 1221 1222 1223 1224 1225 1226 1227 1228 1229 1230 1231 1232 1233 1234 1235 1236 1237 1238 1239 1240 1241 1242 1243 1244 1245 1246 1247 1248 1249 1250 1251 1252 1253 1254 1255 1256 1257 1258 1259 1260 1261 1262 1263 1264 1265 1266 1267 1268 1269 1270 1271 1272 1273 1274 1275 1276 1277 1278 1279 1280 1281 1282 1283 1284 1285 1286 1287 1288 1289 1290 1291 1292 1293 1294 1295 1296 1297 1298 1299 1300 1301 1302 1303 1304 1305 1306 1307 1308 1309 1310 1311 1312 1313 1314 1315 1316 1317 1318 1319 1320 1321 1322 1323 1324 1325 1326 1327 1328 1329 1330 1331 1332 1333 1334 1335 1336 1337 1338 1339 1340 1341 1342 1343 1344 1345 1346 1347 1348 1349 1350 1351 1352 1353 1354 1355 1356 1357 1358 1359 1360 1361 1362 1363 1364 1365 1366 1367 1368 1369 1370 1371 1372 1373 1374 1375 1376 1377 1378 1379 1380 1381 1382 1383 1384 1385 1386 1387 1388 1389 1390 1391 1392 1393 1394 1395 1396 1397 1398 1399 1400 1401 1402 1403 1404 1405 1406 1407 1408 1409 1410 1411 1412 1413 1414 1415 1416 1417 1418 1419 1420 1421 1422 1423 1424 1425 1426 1427 1428 1429 1430 1431 1432 1433 1434 1435 1436 1437 1438 1439 1440 1441 1442 1443 1444 1445 1446 1447 1448 1449 1450 1451 1452 1453 1454 1455 1456 1457 1458 1459 1460 1461 1462 1463 1464 1465 1466 1467 1468 1469 1470 1471 1472 1473 1474 1475 1476 1477 1478 1479 1480 1481 1482 1483 1484 1485 1486 1487 1488 1489 1490 1491 1492 1493 1494 1495 1496 1497 1498 1499 1500 1501 1502 1503 1504 1505 1506 1507 1508 1509 1510 1511 1512 1513 1514 1515 1516 1517 1518 1519 1520 1521 1522 1523 1524 1525 1526 1527 1528 1529 1530 1531 1532 1533 1534 1535 1536 1537 1538 1539 1540 1541 1542 1543 1544 1545 1546 1547 1548 1549 1550 1551 1552 1553 1554 | # coding=utf-8 import logging import typing from calendar import timegm from itertools import chain from time import gmtime, strftime, strptime from emiz.validator import Validator, valid_bool, valid_float, valid_int, valid_positive_int, valid_str EPOCH_DELTA = 1306886400 LOGGER = logging.getLogger('EMIZ').getChild(__name__) validator_group_or_unit_name = Validator(_type=str, _regex=r'[a-zA-Z0-9\_\-\#]+', exc=ValueError, logger=LOGGER) class BaseMissionObject: def __init__(self, mission_dict: dict, l10n: dict): super().__init__() if not isinstance(mission_dict, dict): raise TypeError('mission_dict should be an dict, got: {}'.format(type(mission_dict))) if not isinstance(l10n, dict): raise TypeError('l10n should be an dict, got: {}'.format(type(l10n))) self.d = mission_dict self.l10n = l10n self.weather = None self.blue_coa = None self.red_coa = None self.ground_control = None self._countries_by_name = {} self._countries_by_id = {} def get_country_by_name(self, country_name): valid_str.validate(country_name, 'get_country_by_name', exc=ValueError) if country_name not in self._countries_by_name.keys(): for country in self.countries: assert isinstance(country, Country) if country.country_name == country_name: self._countries_by_name[country_name] = country return country raise ValueError(country_name) else: return self._countries_by_name[country_name] def get_country_by_id(self, country_id): valid_positive_int.validate(country_id, 'get_country_by_id') if country_id not in self._countries_by_id.keys(): for country in self.countries: assert isinstance(country, Country) if country.country_id == country_id: self._countries_by_id[country_id] = country return country raise ValueError(country_id) else: return self._countries_by_id[country_id] def get_groups_from_category(self, category): Mission.validator_group_category.validate(category, 'get_groups_from_category') for group in self.groups: if group.group_category == category: yield group def get_units_from_category(self, category): Mission.validator_group_category.validate(category, 'get_units_from_category') for unit in self.units: if unit.group_category == category: yield unit def get_group_by_id(self, group_id): valid_positive_int.validate(group_id, 'get_group_by_id', exc=ValueError) for group in self.groups: assert isinstance(group, Group) if group.group_id == group_id: return group return None def get_clients_groups(self) -> typing.Generator['FlyingUnit', None, None]: for group in self.groups: assert isinstance(group, Group) if group.group_is_client_group: yield group def get_group_by_name(self, group_name): valid_str.validate(group_name, 'get_group_by_name') for group in self.groups: assert isinstance(group, Group) if group.group_name == group_name: return group return None def get_unit_by_name(self, unit_name): valid_str.validate(unit_name, 'get_unit_by_name') for unit in self.units: assert isinstance(unit, BaseUnit) if unit.unit_name == unit_name: return unit return None def get_unit_by_id(self, unit_id): valid_positive_int.validate(unit_id, 'get_unit_by_id') for unit in self.units: assert isinstance(unit, BaseUnit) if unit.unit_id == unit_id: return unit return None @property def units(self): for group in self.groups: for unit in group.units: assert isinstance(unit, BaseUnit) yield unit @property def groups(self): for country in self.countries: for group in country.groups: assert isinstance(group, Group) yield group @property def next_group_id(self): ids = set() for group in chain(self.blue_coa.groups, self.red_coa.groups): assert isinstance(group, Group) id_ = group.group_id if id_ in ids: raise IndexError(group.group_name) ids.add(id_) return max(ids) + 1 @property def next_unit_id(self): ids = set() for unit in chain(self.blue_coa.units, self.red_coa.units): assert isinstance(unit, BaseUnit) id_ = unit.unit_id if id_ in ids: raise IndexError(unit.unit_name) ids.add(id_) return max(ids) + 1 @property def coalitions(self): for coalition in [self.blue_coa, self.red_coa]: assert isinstance(coalition, Coalition) yield coalition @property def countries(self): for coalition in self.coalitions: for country in coalition.countries: assert isinstance(country, Country) yield country @property def mission_start_time(self): return self.d['start_time'] @mission_start_time.setter def mission_start_time(self, value): Mission.validator_start_time.validate(value, 'start_time') self.d['start_time'] = value @property def mission_start_time_as_date(self): return strftime('%d/%m/%Y %H:%M:%S', gmtime(EPOCH_DELTA + self.mission_start_time)) @mission_start_time_as_date.setter def mission_start_time_as_date(self, value): Mission.validator_start_date.validate(value, 'start_time_as_date') self.mission_start_time = timegm(strptime(value, '%d/%m/%Y %H:%M:%S')) - EPOCH_DELTA @property def _sortie_name_key(self): return self.d['sortie'] @property def sortie_name(self): return self.l10n[self._sortie_name_key] @sortie_name.setter def sortie_name(self, value): valid_str.validate(value, 'sortie name') self.l10n[self._sortie_name_key] = value class Mission(BaseMissionObject): validator_start_time = Validator( _type=int, _min=0, exc=ValueError, logger=LOGGER ) validator_start_date = Validator( _type=str, _regex=r'^(?=\d)(?:(?:31(?!.(?:0?[2469]|11))|(?:30|29)(?!.0?2)|29' r'(?=.0?2.(?:(?:(?:1[6-9]|[2-9]\d)?(?:0[48]|[2468][048]|[13579][26])' r'|(?:(?:16|[2468][048]|[3579][26])00)))(?:\x20|$))|(?:2[0-8]|1\d|0?[1-9]))' r'([-./])(?:1[012]|0?[1-9])\1(?:1[6-9]|[2-9]\d)?\d\d(?:(?=\x20\d)\x20|$))?' r'(((0?[1-9]|1[012])(:[0-5]\d){0,2}(\x20[AP]M))|([01]\d|2[0-3])' r'(:[0-5]\d){1,2})?$', exc=ValueError, logger=LOGGER ) validator_heading = Validator( _type=int, _min=0, _max=359, exc=ValueError, logger=LOGGER ) validator_group_category = Validator( _type=str, _in_list=['helicopter', 'ship', 'plane', 'vehicle'], exc=ValueError, logger=LOGGER) valid_group_categories = ('helicopter', 'plane', 'ship', 'vehicle') def __init__(self, mission_dict, l10n): super().__init__(mission_dict, l10n) self.weather = Weather(self.d, l10n) self.blue_coa = Coalition(self.d, l10n, 'blue') self.red_coa = Coalition(self.d, l10n, 'red') self.ground_control = GroundControl(self.d, l10n) def __repr__(self): return 'Mission({})'.format(self.d) def farps(self) -> typing.Generator['Static', None, None]: for coa in [self.blue_coa, self.red_coa]: for farp in coa.farps: yield farp # noinspection PyProtectedMember class Coalition(BaseMissionObject): def __init__(self, mission_dict, ln10, coa_color): super().__init__(mission_dict, ln10) self.coa_color = coa_color self._countries = {} def __repr__(self): return 'Coalition({}, {})'.format(self._section_coalition, self.coa_color) def __eq__(self, other): if not isinstance(other, Coalition): raise ValueError('"other" must be an Coalition instance; got: {}'.format(type(other))) return self._section_coalition == other._section_coalition @property def _section_coalition(self): return self.d['coalition'][self.coa_color] @property def _section_bullseye(self): return self._section_coalition['bullseye'] @property def bullseye_x(self): return self._section_bullseye['x'] @property def bullseye_y(self): return self._section_bullseye['y'] @property def bullseye_position(self): return self.bullseye_x, self.bullseye_y @property def _section_nav_points(self): return self._section_coalition['nav_points'] @property def coalition_name(self): return self._section_coalition['name'] @property def _section_country(self): return self._section_coalition['country'] @property def countries(self) -> typing.Generator['Country', None, None]: for k in self._section_country: if k not in self._countries.keys(): country = Country(self.d, self.l10n, self.coa_color, k) self._countries[k] = country self._countries_by_id[country.country_id] = country self._countries_by_name[country.country_name] = country yield self._countries[k] def get_country_by_name(self, country_name) -> 'Country': valid_str.validate(country_name, 'get_country_by_name', exc=ValueError) if country_name not in self._countries_by_name.keys(): for country in self.countries: assert isinstance(country, Country) if country.country_name == country_name: return country raise ValueError(country_name) else: return self._countries_by_name[country_name] def get_country_by_id(self, country_id) -> 'Country': valid_positive_int.validate(country_id, 'get_country_by_id', exc=ValueError) if country_id not in self._countries_by_id.keys(): for country in self.countries: assert isinstance(country, Country) if country.country_id == country_id: return country raise ValueError(country_id) else: return self._countries_by_id[country_id] @property def groups(self) -> typing.Generator['Group', None, None]: for country in self.countries: assert isinstance(country, Country) for group in country.groups: assert isinstance(group, Group) yield group @property def statics(self) -> typing.Generator['Static', None, None]: for country in self.countries: assert isinstance(country, Country) for static in country.statics: assert isinstance(static, Static) yield static @property def farps(self) -> typing.Generator['Static', None, None]: for country in self.countries: assert isinstance(country, Country) for static in country.statics: assert isinstance(static, Static) if static.static_is_farp: yield static def get_groups_from_category(self, category) -> typing.Generator['Group', None, None]: Mission.validator_group_category.validate(category, 'get_groups_from_category') for group in self.groups: assert isinstance(group, Group) if group.group_category == category: yield group @property def units(self) -> typing.Generator['BaseUnit', None, None]: for group in self.groups: assert isinstance(group, Group) for unit in group.units: yield unit def get_units_from_category(self, category) -> typing.Generator['BaseUnit', None, None]: Mission.validator_group_category.validate(category, 'group category') for unit in self.units: assert isinstance(unit, BaseUnit) if unit.group_category == category: yield unit def get_group_by_id(self, group_id) -> 'Group': valid_positive_int.validate(group_id, 'get_group_by_id') for group in self.groups: assert isinstance(group, Group) if group.group_id == group_id: return group def get_group_by_name(self, group_name) -> 'Group': valid_str.validate(group_name, 'get_group_by_name') for group in self.groups: assert isinstance(group, Group) if group.group_name == group_name: return group def get_unit_by_name(self, unit_name) -> 'BaseUnit': valid_str.validate(unit_name, 'get_unit_by_name') for unit in self.units: assert isinstance(unit, BaseUnit) if unit.unit_name == unit_name: return unit def get_unit_by_id(self, unit_id) -> 'BaseUnit': valid_positive_int.validate(unit_id, 'get_unit_by_id') for unit in self.units: assert isinstance(unit, BaseUnit) if unit.unit_id == unit_id: return unit class Trig(BaseMissionObject): def __init__(self, mission_dict, l10n): super().__init__(mission_dict, l10n) @property def _section_trig(self): return self.d['trig'] class Result(BaseMissionObject): def __init__(self, mission_dict, l10n): super().__init__(mission_dict, l10n) @property def _section_result(self): return self.d['result'] class GroundControl(BaseMissionObject): validator_commander = Validator(_type=int, _min=0, _max=100, exc=ValueError, logger=LOGGER) def __init__(self, mission_dict, l10n): super().__init__(mission_dict, l10n) def __repr__(self): return 'GroundControl({})'.format(self._section_ground_control) @property def _section_ground_control(self): return self.d['groundControl'] @property def _section_ground_control_roles(self): return self.d['groundControl']['roles'] @property def pilots_control_vehicles(self): return self._section_ground_control['isPilotControlVehicles'] @pilots_control_vehicles.setter def pilots_control_vehicles(self, value): valid_bool.validate(value, 'pilots_control_vehicles') self._section_ground_control['isPilotControlVehicles'] = value @property def _section_artillery_commander(self): return self._section_ground_control_roles['artillery_commander'] @property def artillery_commander_red(self): return self._section_artillery_commander['red'] @artillery_commander_red.setter def artillery_commander_red(self, value): self.validator_commander.validate(value, 'artillery_commander_red') self._section_artillery_commander['red'] = value @property def instructor_blue(self): return self._section_instructor['blue'] @instructor_blue.setter def instructor_blue(self, value): self.validator_commander.validate(value, 'instructor_blue') self._section_instructor['blue'] = value @property def instructor_red(self): return self._section_instructor['red'] @instructor_red.setter def instructor_red(self, value): self.validator_commander.validate(value, 'instructor_red') self._section_instructor['red'] = value @property def _section_observer(self): return self._section_ground_control_roles['observer'] @property def observer_blue(self): return self._section_observer['blue'] @observer_blue.setter def observer_blue(self, value): self.validator_commander.validate(value, 'observer_blue') self._section_observer['blue'] = value @property def observer_red(self): return self._section_observer['red'] @observer_red.setter def observer_red(self, value): self.validator_commander.validate(value, 'observer_red') self._section_observer['red'] = value @property def _section_forward_observer(self): return self._section_ground_control_roles['forward_observer'] @property def forward_observer_blue(self): return self._section_forward_observer['blue'] @forward_observer_blue.setter def forward_observer_blue(self, value): self.validator_commander.validate(value, 'forward_observer_blue') self._section_forward_observer['blue'] = value @property def forward_observer_red(self): return self._section_forward_observer['red'] @forward_observer_red.setter def forward_observer_red(self, value): self.validator_commander.validate(value, 'forward_observer_red') self._section_forward_observer['red'] = value @property def artillery_commander_blue(self): return self._section_artillery_commander['blue'] @artillery_commander_blue.setter def artillery_commander_blue(self, value): self.validator_commander.validate(value, 'artillery_commander_blue') self._section_artillery_commander['blue'] = value @property def _section_instructor(self): return self._section_ground_control_roles['instructor'] # noinspection PyProtectedMember class Weather(BaseMissionObject): validator_precipitations = Validator(_type=int, _min=0, _max=4, exc=ValueError, logger=LOGGER) validator_cloud_density = Validator(_type=int, _min=0, _max=10, exc=ValueError, logger=LOGGER) validator_cloud_thickness = Validator(_type=int, _min=200, _max=2000, exc=ValueError, logger=LOGGER) validator_cloud_base = Validator(_type=int, _min=300, _max=5000, exc=ValueError, logger=LOGGER) validator_fog_visibility = Validator(_type=int, _min=0, _max=6000, exc=ValueError, logger=LOGGER) validator_visibility = Validator(_type=int, _min=0, _max=80000, exc=ValueError, logger=LOGGER) validator_fog_thickness = Validator(_type=int, _min=0, _max=1000, exc=ValueError, logger=LOGGER) validator_qnh = Validator(_type=int, _min=720, _max=790, exc=ValueError, logger=LOGGER) validator_temperature = Validator(_type=int, _min=-50, _max=50, exc=ValueError, logger=LOGGER) validator_temp_spring_or_fall = Validator(_type=int, _min=-10, _max=30, exc=ValueError, logger=LOGGER) validator_temp_winter = Validator(_type=int, _min=-50, _max=15, exc=ValueError, logger=LOGGER) validator_temp_summer = Validator(_type=int, _min=5, _max=50, exc=ValueError, logger=LOGGER) seasons_enum = { 1: { 'name': 'summer', 'temp_validator': validator_temp_summer, }, 2: { 'name': 'winter', 'temp_validator': validator_temp_winter, }, 3: { 'name': 'spring', 'temp_validator': validator_temp_spring_or_fall, }, 4: { 'name': 'fall', 'temp_validator': validator_temp_spring_or_fall, }, 'summer': 1, 'winter': 2, 'spring': 3, 'fall': 4, } validator_season_name = Validator(_type=str, _in_list=['summer', 'winter', 'fall', 'spring'], exc=ValueError, logger=LOGGER) validator_season_code = Validator(_type=int, _min=1, _max=4, exc=ValueError, logger=LOGGER) validator_turbulence = Validator(_type=int, _min=0, _max=60, exc=ValueError, logger=LOGGER) validator_atmo_type = Validator(_type=int, _min=0, _max=1, exc=ValueError, logger=LOGGER) validator_wind_speed = Validator(_type=int, _min=0, _max=50, exc=ValueError, logger=LOGGER) def __init__(self, mission_dict, l10n): super().__init__(mission_dict, l10n) def __repr__(self): return 'Weather({})'.format(self._section_weather) def __eq__(self, other): if not isinstance(other, Weather): raise ValueError('"other" must be an Weather instance; got: {}'.format(type(other))) return self._section_weather == other._section_weather def get_season_code_from_name(self, season_name): self.validator_season_name.validate(season_name, 'get_season_code_from_name') return self.seasons_enum[season_name] @property def _section_wind_at_ground_level(self): return self._section_wind['atGround'] @property def turbulence_at_ground_level(self): return self._section_weather['groundTurbulence'] @turbulence_at_ground_level.setter def turbulence_at_ground_level(self, value): self.validator_turbulence.validate(value, 'turbulence_at_ground_level') self._section_weather['groundTurbulence'] = value @property def wind_at_ground_level_speed(self): return self._section_wind_at_ground_level['speed'] @wind_at_ground_level_speed.setter def wind_at_ground_level_speed(self, value): self.validator_wind_speed.validate(value, 'wind_at_ground_level_speed') self._section_wind_at_ground_level['speed'] = value @property def _section_fog(self): return self._section_weather['fog'] @property def fog_thickness(self): return self._section_fog['thickness'] @fog_thickness.setter def fog_thickness(self, value): self.validator_fog_thickness.validate(value, 'fog_thickness') self._section_fog['thickness'] = value @property def fog_visibility(self): return self._section_fog['visibility'] @fog_visibility.setter def fog_visibility(self, value): self.validator_fog_visibility.validate(value, 'fog_visibility') self._section_fog['visibility'] = value @property def fog_enabled(self): return self._section_weather['enable_fog'] @fog_enabled.setter def fog_enabled(self, value): valid_bool.validate(value, 'enable_fog') self._section_weather['enable_fog'] = value @property def _section_visibility(self): return self._section_weather['visibility'] @property def visibility(self): return self._section_visibility['distance'] @visibility.setter def visibility(self, value): self.validator_visibility.validate(value, 'visibility') self._section_visibility['distance'] = value @property def precipitations(self): return self._section_clouds['iprecptns'] @precipitations.setter def precipitations(self, value): self.validator_precipitations.validate(value, 'precipitations') if value > 0 and self.cloud_density <= 4: raise ValueError('No rain or snow if cloud density is less than 5') if value in [2, 4] and self.cloud_density <= 8: raise ValueError('No thunderstorm or snowstorm if cloud density is less than 9') if value > 2 and self.temperature > 0: raise ValueError('No snow with temperature over 0; use rain or thunderstorm instead') if value in [1, 2] and self.temperature < 0: raise ValueError( 'No rain or thunderstorm if temperature is below 0; use snow or snowstorm instead') self._section_clouds['iprecptns'] = value @property def wind_at8000_dir(self): return self._section_wind_at8000['speed'] @wind_at8000_dir.setter def wind_at8000_dir(self, value): Mission.validator_heading.validate(value, 'wind_at8000_dir') self._section_wind_at8000['dir'] = value @property def _section_weather(self): return self.d['weather'] @property def temperature(self): return self._section_season['temperature'] @temperature.setter def temperature(self, value): self.validator_temperature.validate(value, 'temperature') self._section_season['temperature'] = value # if value > 0 and self.precipitations > 2: # self.precipitations -= 2 # if value < 0 < self.precipitations < 3: # PyKek # self.precipitations += 2 @property def _section_wind_at8000(self): return self._section_wind['at8000'] @property def wind_at_ground_level_dir(self): return self._section_wind_at_ground_level['dir'] @wind_at_ground_level_dir.setter def wind_at_ground_level_dir(self, value): Mission.validator_heading.validate(value, 'wind_at_ground_level_dir') self._section_wind_at_ground_level['dir'] = value @property def _section_wind(self): return self._section_weather['wind'] @property def _section_wind_at2000(self): return self._section_wind['at2000'] @property def season_name(self): return self.seasons_enum[self.season_code]['name'] @property def qnh(self): return self._section_weather['qnh'] @qnh.setter def qnh(self, value): self.validator_qnh.validate(value, 'qnh') self._section_weather['qnh'] = value @property def wind_at2000_speed(self): return self._section_wind_at2000['speed'] @wind_at2000_speed.setter def wind_at2000_speed(self, value): self.validator_wind_speed.validate(value, 'wind_at2000_speed') self._section_wind_at2000['speed'] = value @property def wind_at2000_dir(self): return self._section_wind_at2000['dir'] @wind_at2000_dir.setter def wind_at2000_dir(self, value): Mission.validator_heading.validate(value, 'wind_at2000_dir') self._section_wind_at2000['dir'] = value @property def _section_season(self): return self._section_weather['season'] @property def atmosphere_type(self): return self._section_weather['atmosphere_type'] @atmosphere_type.setter def atmosphere_type(self, value): self.validator_atmo_type.validate(value, 'atmosphere_type') self._section_weather['atmosphere_type'] = value @property def season_code(self): return self._section_season['iseason'] @season_code.setter def season_code(self, value): self.validator_season_code.validate(value, 'season') self._section_season['iseason'] = value if self.temperature < self.seasons_enum[value]['temp_validator'].min: self.temperature = self.seasons_enum[value]['temp_validator'].min if self.temperature > self.seasons_enum[value]['temp_validator'].max: self.temperature = self.seasons_enum[value]['temp_validator'].max @property def _section_clouds(self): return self._section_weather['clouds'] @property def cloud_thickness(self): return self._section_clouds['thickness'] @cloud_thickness.setter def cloud_thickness(self, value): self.validator_cloud_thickness.validate(value, 'cloud_thickness') self._section_clouds['thickness'] = value @property def cloud_base(self): return self._section_clouds['base'] @cloud_base.setter def cloud_base(self, value): self.validator_cloud_base.validate(value, 'cloud_base') self._section_clouds['base'] = value @property def cloud_density(self): return self._section_clouds['density'] @cloud_density.setter def cloud_density(self, value): self.validator_cloud_density.validate(value, 'cloud_density') self._section_clouds['density'] = value @property def wind_at8000_speed(self): return self._section_wind_at8000['speed'] @wind_at8000_speed.setter def wind_at8000_speed(self, value): self.validator_wind_speed.validate(value, 'wind_at8000_speed') self._section_wind_at8000['speed'] = value # noinspection PyProtectedMember class Country(Coalition): def __init__(self, mission_dict, l10n, coa_color, country_index): super().__init__(mission_dict, l10n, coa_color) self.__groups = { 'helicopter': {}, 'plane': {}, 'vehicle': {}, 'ship': {}, } self.country_index = country_index self.__static = {} def __repr__(self): return 'Country({}, {}, {})'.format(self._section_country, self.coa_color, self.country_index) def __eq__(self, other): if not isinstance(other, Country): raise ValueError('"other" must be an Country instance; got: {}'.format(type(other))) return self._section_country == other._section_country @property def _section_this_country(self): return self._section_coalition['country'][self.country_index] @property def country_id(self): return self._section_this_country['id'] @property def country_name(self): return self._section_this_country['name'] @property def groups(self) -> typing.Generator['Group', None, None]: for group_category in Mission.valid_group_categories: if group_category in self._section_this_country.keys(): for group_index in self._section_this_country[group_category]['group']: if group_index not in self.__groups[group_category]: self.__groups[group_category][group_index] = Group(self.d, self.l10n, self.coa_color, self.country_index, group_category, group_index) yield self.__groups[group_category][group_index] @property def statics(self) -> typing.Generator['Static', None, None]: if 'static' in self._section_this_country.keys(): for static_index in self._section_this_country['static']['group']: if static_index not in self.__static: self.__static[static_index] = Static(self.d, self.l10n, self.coa_color, self.country_index, static_index) yield self.__static[static_index] def get_groups_from_category(self, category) -> typing.Generator['Group', None, None]: Mission.validator_group_category.validate(category, 'get_groups_from_category') for group in self.groups: assert isinstance(group, Group) if group.group_category == category: yield group def get_group_by_id(self, group_id) -> 'Group': for group in self.groups: assert isinstance(group, Group) if group.group_id == group_id: return group def get_group_by_name(self, group_name) -> 'Group': for group in self.groups: assert isinstance(group, Group) if group.group_name == group_name: return group @property def units(self) -> typing.Generator['BaseUnit', None, None]: for group in self.groups: assert isinstance(group, Group) for unit in group.units: yield unit def get_unit_by_name(self, unit_name) -> 'BaseUnit': for unit in self.units: assert isinstance(unit, BaseUnit) if unit.unit_name == unit_name: return unit def get_unit_by_id(self, unit_id) -> 'BaseUnit': for unit in self.units: assert isinstance(unit, BaseUnit) if unit.unit_id == unit_id: return unit def get_units_from_category(self, category) -> typing.Generator['BaseUnit', None, None]: Mission.validator_group_category.validate(category, 'group category') for unit in self.units: assert isinstance(unit, BaseUnit) if unit.group_category == category: yield unit class Static(Country): def __init__(self, mission_dict, l10n, coa_color, country_index, static_index): super(Static, self).__init__(mission_dict, l10n, coa_color, country_index) self.static_index = static_index @property def static_id(self): return self._section_static['groupId'] @static_id.setter def static_id(self, value): valid_int.validate(value, 'groupId') self._section_static['groupId'] = value @property def _section_static(self): return self._section_this_country['static']['group'][self.static_index] @property def _static_name_key(self): return self._section_static['name'] @property def static_name(self): return self.l10n[self._static_name_key] @static_name.setter def static_name(self, value): validator_group_or_unit_name.validate(value, 'group name') self.l10n[self._static_name_key] = value @property def static_category(self): return self._section_static['units'][1]['category'] @property def static_is_farp(self): return self.static_category == 'Heliports' @property def static_position(self): unit = self._section_static['units'][1] return unit['x'], unit['y'] # noinspection PyProtectedMember class Group(Country): attribs = ('group_category', 'group_index', 'group_hidden', 'group_start_time', '_group_name_key') class Route: class Point: def __init__(self, parent_route): assert isinstance(parent_route, Group.Route) self.parent_route = parent_route def __repr__(self): return 'Route({})'.format(self.parent_route.parent_group.group_name) def __init__(self, parent_group): assert isinstance(parent_group, Group) self.parent_group = parent_group def __repr__(self): return 'Route({})'.format(self.parent_group.group_name) @property def _section_route(self): return self.parent_group._section_group['route']['points'] @property def points(self): raise NotImplementedError('uh') validator_group_route = Validator(_type=Route, exc=ValueError, logger=LOGGER) units_class_enum = None def __init__(self, mission_dict, l10n, coa_color, country_index, group_category, group_index): super().__init__(mission_dict, l10n, coa_color, country_index) self.group_category = group_category self.group_index = group_index self.__group_route = None self.__units = {} self.units_class_enum = { 'helicopter': Helicopter, 'plane': Plane, 'ship': Ship, 'vehicle': Vehicle, } def __repr__(self): return 'Group({}, {}, {}, {}, {})'.format(self._section_group, self.coa_color, self.country_index, self.group_category, self.group_index) def __eq__(self, other): if not isinstance(other, Group): raise ValueError( '"other" must be an AbstractUnit instance; got: {}'.format(type(other))) return self._section_group == other._section_group # noinspection PyTypeChecker @property def group_route(self) -> 'Group.Route': # TODO if self.__group_route is None: self.__group_route = Group.Route(self) return self.__group_route @group_route.setter def group_route(self, value): self.validator_group_route.validate(value, 'group_route') self.__group_route = value @property def _section_group(self): return self._section_this_country[self.group_category]['group'][self.group_index] @property def _group_name_key(self): return self._section_group['name'] @property def group_name(self): return self.l10n[self._group_name_key] @group_name.setter def group_name(self, value): validator_group_or_unit_name.validate(value, 'group name') self.l10n[self._group_name_key] = value @property def group_hidden(self): return self._section_group['hidden'] @group_hidden.setter def group_hidden(self, value): valid_bool.validate(value, 'property "hidden" for group') self._section_group['hidden'] = value @property def group_id(self): return self._section_group['groupId'] @group_id.setter def group_id(self, value): valid_int.validate(value, 'groupId') self._section_group['groupId'] = value @property def group_start_delay(self): return self._section_group['start_time'] @group_start_delay.setter def group_start_delay(self, value): valid_int.validate(value, 'group_start_delay') if value < 0: raise ValueError(self.group_name) self._section_group['start_time'] = value @property def group_start_time(self): return self.group_start_delay + self.mission_start_time @group_start_time.setter def group_start_time(self, value): valid_int.validate(value, 'group_start_time') self.group_start_delay = value - self.mission_start_time @property def group_start_time_as_date(self): return strftime('%d/%m/%Y %H:%M:%S', gmtime(EPOCH_DELTA + self.group_start_time)) @group_start_time_as_date.setter def group_start_time_as_date(self, value): Mission.validator_start_date.validate(value, 'start_time_as_date') self.group_start_time = timegm(strptime(value, '%d/%m/%Y %H:%M:%S')) - EPOCH_DELTA @property def units(self): for unit_index in self._section_group['units']: if unit_index not in self.__units.keys(): self.__units[unit_index] = self.units_class_enum[self.group_category](self.d, self.l10n, self.coa_color, self.country_index, self.group_category, self.group_index, unit_index) yield self.__units[unit_index] @property def first_unit(self) -> 'BaseUnit': return list(self.units)[0] def group_size(self) -> int: return len(list(self.units)) def get_unit_by_name(self, unit_name): for unit in self.units: assert isinstance(unit, BaseUnit) if unit.unit_name == unit_name: return unit return None def get_unit_by_id(self, unit_id): for unit in self.units: assert isinstance(unit, BaseUnit) if unit.unit_id == unit_id: return unit return None def get_unit_by_index(self, unit_index): if unit_index in self._section_group['units'].keys(): if unit_index not in self.__units.keys(): self.__units[unit_index] = self.units_class_enum[self.group_category](self.d, self.l10n, self.coa_color, self.country_index, self.group_category, self.group_index, unit_index) return self.__units[unit_index] return None @property def group_is_client_group(self): # TODO create test first_unit = self.get_unit_by_index(1) assert isinstance(first_unit, BaseUnit) return first_unit.skill == 'Client' @property def group_start_position(self): return self.group_route._section_route[1]['action'] # noinspection PyProtectedMember class BaseUnit(Group): validator_skill = Validator(_type=str, _in_list=['Average', 'Good', 'High', 'Excellent', 'Random', 'Client', 'Player'], exc=ValueError, logger=LOGGER) validator_unit_types = Validator(_type=str, _in_list=[], exc=ValueError, logger=LOGGER) def __init__(self, mission_dict, l10n, coa_color, country_index, group_category, group_index, unit_index): super().__init__(mission_dict, l10n, coa_color, country_index, group_category, group_index) self.unit_index = unit_index def __repr__(self): return '{}({}, {}, {}, {}, {}, {})'.format(self.__class__.__name__, self._section_unit, self.coa_color, self.country_index, self.group_category, self.group_index, self.unit_index) @property def _section_unit(self): return self._section_group['units'][self.unit_index] @property def _unit_name_key(self): return self._section_unit['name'] @property def unit_name(self): return self.l10n[self._unit_name_key] @unit_name.setter def unit_name(self, value): validator_group_or_unit_name.validate(value, 'unit name') self.l10n[self._unit_name_key] = value @property def skill(self): return self._section_unit['skill'] @skill.setter def skill(self, value): self.validator_skill.validate(value, 'unit skill') self._section_unit['skill'] = value @property def speed(self): return self._section_unit['speed'] @speed.setter def speed(self, value): valid_float.validate(value, 'unit speed') self._section_unit['speed'] = value @property def unit_type(self): return self._section_unit['type'] @unit_type.setter def unit_type(self, value): self.validator_unit_types.validate(value, 'unit type') self._section_unit['type'] = value @property def unit_id(self): return self._section_unit['unitId'] @unit_id.setter def unit_id(self, value): valid_int.validate(value, 'unitId') self._section_unit['unitId'] = value @property def unit_pos_x(self): return float(self._section_unit['x']) @unit_pos_x.setter def unit_pos_x(self, value): valid_float.validate(value, 'unit position X coordinate') self._section_unit['x'] = value @property def unit_pos_y(self): return float(self._section_unit['y']) @unit_pos_y.setter def unit_pos_y(self, value): valid_float.validate(value, 'unit position Y coordinate') self._section_unit['y'] = value @property def unit_position(self): return self.unit_pos_x, self.unit_pos_y @unit_position.setter def unit_position(self, value): self.unit_pos_x, self.unit_pos_y = value @property def heading(self): return self._section_unit['heading'] @heading.setter def heading(self, value): Mission.validator_heading.validate(value, 'unit heading') self._section_unit['heading'] = value @property def radio_presets(self) -> typing.Generator['FlyingUnit.RadioPresets', None, None]: raise TypeError('unit #{}: {}'.format(self.unit_id, self.unit_name)) @property def has_radio_presets(self): return all([self.skill == 'Client', self.unit_type in FlyingUnit.RadioPresets.radio_enum.keys()]) def __eq__(self, other): if not isinstance(other, BaseUnit): raise ValueError( '"other" must be an AbstractUnit instance; got: {}'.format(type(other))) return self._section_unit == other._section_unit class FlyingUnit(BaseUnit): validator_board_number = Validator(_type=str, _regex=r'[0-9]{3}', exc=ValueError, logger=LOGGER) class RadioPresets: radio_enum = { 'Ka-50': { 1: { 'radio_name': 'R828', 'min': 20, 'max': 59.9, 'channels_qty': 10, }, 2: { 'radio_name': 'ARK22', 'min': 0.15, 'max': 1.75, 'channels_qty': 16, }, }, 'Mi-8MT': { 1: { 'radio_name': 'R863', 'min': 100, 'max': 399.9, 'channels_qty': 20, }, 2: { 'radio_name': 'R828', 'min': 20, 'max': 59.9, 'channels_qty': 10, }, }, 'UH-1H': { 1: { 'radio_name': 'ARC51', 'min': 225, 'max': 399.97, 'channels_qty': 20, }, }, 'F-86F Sabre': { 1: { 'radio_name': 'ARC-27', 'min': 225, 'max': 399.9, 'channels_qty': 18, }, }, 'M-2000C': { 1: { 'radio_name': 'UHF', 'min': 225, 'max': 400, 'channels_qty': 20, }, 2: { 'radio_name': 'V/UHF', 'min': 118, 'max': 400, 'channels_qty': 20, }, }, 'MiG-21Bis': { 1: { 'radio_name': 'R-832', 'min': 80, 'max': 399.9, 'channels_qty': 20, }, }, 'P-51D': { 1: { 'radio_name': 'SCR552', 'min': 100, 'max': 156, 'channels_qty': 4, }, }, 'TF-51D': { 1: { 'radio_name': 'SCR552', 'min': 100, 'max': 156, 'channels_qty': 4, }, }, 'SpitfireLFMkIX': { 1: { 'radio_name': 'SCR522', 'min': 100, 'max': 156, 'channels_qty': 4, }, }, 'Bf-109K-4': { 1: { 'radio_name': 'FuG 16 ZY', 'min': 38, 'max': 156, 'channels_qty': 5, }, }, 'FW-190D9': { 1: { 'radio_name': 'FuG 16', 'min': 38.4, 'max': 42.4, 'channels_qty': 4, }, }, 'SA342L': { 1: { 'radio_name': 'FM Radio', 'min': 30, 'max': 87.975, 'channels_qty': 8, }, }, 'SA342M': { 1: { 'radio_name': 'FM Radio', 'min': 30, 'max': 87.975, 'channels_qty': 8, }, }, 'SA342Mistral': { 1: { 'radio_name': 'FM Radio', 'min': 30, 'max': 87.975, 'channels_qty': 8, }, }, } def __init__(self, parent_unit, radio_num): assert isinstance(parent_unit, FlyingUnit) self.parent_unit = parent_unit self.radio_num = radio_num def __eq__(self, other): if not isinstance(other, FlyingUnit.RadioPresets): raise Exception('cannot compare RadioPreset instance with other object of type {}'.format(type(other))) assert isinstance(other, FlyingUnit.RadioPresets) if not self.radio_name == other.radio_name: return False for channel, frequency in self.channels: if not frequency == other.get_frequency(channel): return False return True @property def radio_name(self): return self.radio_enum[self.parent_unit.unit_type][self.radio_num]['radio_name'] @property def channels_qty(self): return self.radio_enum[self.parent_unit.unit_type][self.radio_num]['channels_qty'] @property def min(self): return float(self.radio_enum[self.parent_unit.unit_type][self.radio_num]['min']) @property def max(self): return float(self.radio_enum[self.parent_unit.unit_type][self.radio_num]['max']) @property def _section_radio(self): return self.parent_unit._section_unit['Radio'] @property def _section_channels(self): return self._section_radio[self.radio_num]['channels'] @property def channels(self) -> typing.Generator[tuple, None, None]: for k in self._section_channels: yield (k, float(self._section_channels[k])) def get_frequency(self, channel): valid_positive_int.validate(channel, 'get_frequency') if 1 <= channel <= self.channels_qty: return float(self._section_channels[channel]) else: raise ValueError( 'channel {} for radio {} in aircraft {}'.format(channel, self.radio_name, self.parent_unit.unit_name)) def set_frequency(self, channel, frequency): valid_positive_int.validate(channel, 'set_frequency') valid_float.validate(frequency, 'set_frequency') if 1 <= channel <= self.channels_qty: # noinspection PyTypeChecker if self.min <= frequency <= self.max: self._section_channels[channel] = float(frequency) else: raise ValueError( 'frequency {} for channel {} for radio {} in aircraft {}'.format(frequency, channel, self.radio_name, self.parent_unit.unit_name)) else: raise ValueError( 'channel {} for radio {} in aircraft {}'.format(channel, self.radio_name, self.parent_unit.unit_name)) def __init__(self, mission_dict, l10n, coa_color, country_index, group_category, group_index, unit_index): super().__init__(mission_dict, l10n, coa_color, country_index, group_category, group_index, unit_index) @property def radio_presets(self) -> typing.Generator['FlyingUnit.RadioPresets', None, None]: if self.skill == 'Client' and self.unit_type in FlyingUnit.RadioPresets.radio_enum.keys(): for k in self._section_unit['Radio']: yield FlyingUnit.RadioPresets(self, k) else: raise TypeError('unit #{}: {}'.format(self.unit_id, self.unit_name)) @property def radios(self): try: return self._section_unit['Radio'] except KeyError: raise KeyError(self.unit_type) def get_radio_by_name(self, radio_name): if self.has_radio_presets: for k in FlyingUnit.RadioPresets.radio_enum[self.unit_type].keys(): if radio_name == FlyingUnit.RadioPresets.radio_enum[self.unit_type][k]['radio_name']: return FlyingUnit.RadioPresets(self, k) raise TypeError('{} for aircraft: {}'.format(radio_name, self.unit_type)) else: raise TypeError('unit #{}: {}'.format(self.unit_id, self.unit_name)) def get_radio_by_number(self, radio_number): if self.has_radio_presets: if radio_number in FlyingUnit.RadioPresets.radio_enum[self.unit_type].keys(): return FlyingUnit.RadioPresets(self, radio_number) else: raise TypeError( 'radio number {} for aircraft: {}'.format(radio_number, self.unit_type)) else: raise TypeError('unit #{}: {}'.format(self.unit_id, self.unit_name)) @property def livery(self): return self._section_unit['livery_id'] @livery.setter def livery(self, value): # TODO validate livery_id valid_str.validate(value, 'unit livery') self._section_unit['livery_id'] = value @property def onboard_num(self): return self._section_unit['onboard_num'] @onboard_num.setter def onboard_num(self, value): FlyingUnit.validator_board_number.validate(value, 'unit onboard number') self._section_unit['onboard_num'] = value class Helicopter(FlyingUnit): def __init__(self, mission_dict, l10n, coa_color, country_index, group_category, group_index, unit_index): super().__init__(mission_dict, l10n, coa_color, country_index, group_category, group_index, unit_index) class Plane(FlyingUnit): def __init__(self, mission_dict, l10n, coa_color, country_index, group_category, group_index, unit_index): super().__init__(mission_dict, l10n, coa_color, country_index, group_category, group_index, unit_index) class Vehicle(BaseUnit): def __init__(self, mission_dict, l10n, coa_color, country_index, group_category, group_index, unit_index): super().__init__(mission_dict, l10n, coa_color, country_index, group_category, group_index, unit_index) class Ship(BaseUnit): def __init__(self, mission_dict, l10n, coa_color, country_index, group_category, group_index, unit_index): super().__init__(mission_dict, l10n, coa_color, country_index, group_category, group_index, unit_index)